@@ -7,20 +7,17 @@ linking and assembling.
 import io
 import logging
 import os
-import sys
-import platform
 import stat
 import xml
-from .arch.arch import Architecture
 from .lang.c import preprocess, c_to_ir, COptions
-from .lang.c3 import C3Builder
-from .lang.bf import BrainFuckGenerator
-from .lang.fortran import FortranBuilder
-from .lang.llvmir import LlvmIrFrontend
+from .lang.c3 import c3_to_ir
+from .lang.bf import bf_to_ir
+from .lang.fortran import fortran_to_ir
+from .lang.llvmir import llvm_to_ir
 from .lang.pascal import PascalBuilder
-from .lang.ws import WhitespaceGenerator
+from .lang.ws import ws_to_ir
 from .lang.python import python_to_ir, ir_to_python
-from .irs.wasm import wasm_to_ir, read_wasm
+from .wasm import wasm_to_ir, read_wasm
 from .irutils import verify_module
 from .utils.reporting import DummyReportGenerator, HtmlReportGenerator
 from .opt.transform import DeleteUnusedInstructionsPass
@@ -31,6 +28,7 @@ from .opt import LoadAfterStorePass
 from .opt import CleanPass
 from .opt.mem2reg import Mem2RegPromotor
 from .opt.cjmp import CJumpPass
+from .opt.tailcall import TailCallOptimization
 from .codegen import CodeGenerator
 from .binutils.linker import link
 from .binutils.outstream import BinaryOutputStream, TextOutputStream
@@ -38,47 +36,22 @@ from .binutils.outstream import MasterOutputStream, FunctionOutputStream
 from .binutils.objectfile import ObjectFile, get_object
 from .binutils.debuginfo import DebugAddress, DebugInfo
 from .binutils.disasm import Disassembler
-from .utils.hexfile import HexFile
-from .utils.elffile import ElfFile
-from .utils.exefile import ExeWriter
-from .utils import uboot_image
+from .format.hexfile import HexFile
+from .format.elf import write_elf
+from .format.exefile import ExeWriter
+from .format import uboot_image
 from .build.tasks import TaskError, TaskRunner
 from .build.recipe import RecipeLoader
 from .common import CompilerError, DiagnosticsManager, get_file
-from .arch.target_list import create_arch
+from .arch import get_arch, get_current_arch
 
 # When using 'from ppci.api import *' include the following:
 __all__ = [
     'asm', 'c3c', 'cc', 'link', 'objcopy', 'bfcompile', 'construct',
     'optimize', 'preprocess',
-    'get_arch', 'ir_to_object', 'ir_to_python']
-
-
-def get_arch(arch):
-    """ Try to return an architecture instance.
-
-    Args:
-        arch: can be a string in the form of arch:option1:option2
-
-    .. doctest::
-
-        >>> from ppci.api import get_arch
-        >>> arch = get_arch('msp430')
-        >>> arch
-        msp430-arch
-        >>> type(arch)
-        <class 'ppci.arch.msp430.arch.Msp430Arch'>
-    """
-    if isinstance(arch, Architecture):
-        return arch
-    elif isinstance(arch, str):
-        if ':' in arch:
-            # We have target with options attached
-            l = arch.split(':')
-            return create_arch(l[0], options=tuple(l[1:]))
-        else:
-            return create_arch(arch)
-    raise TaskError('Invalid architecture {}'.format(arch))
+    'get_arch', 'get_current_arch', 'is_platform_supported',
+    'ir_to_object', 'ir_to_python',
+    'bf_to_ir', 'ws_to_ir']
 
 
 def get_reporter(reporter):
@@ -86,7 +59,7 @@ def get_reporter(reporter):
         return DummyReportGenerator()
     elif isinstance(reporter, str):
         if reporter.endswith('.html'):
-            f = open(reporter, 'w')
+            f = open(reporter, 'wt', encoding='utf8')
             r = HtmlReportGenerator(f)
             r.header()
             return r
@@ -102,17 +75,6 @@ def is_platform_supported():
     return get_current_arch() is not None
 
 
-def get_current_arch():
-    """ Try to get the architecture for the current platform """
-    if sys.platform.startswith('win'):
-        machine = platform.machine()
-        if machine == 'AMD64':
-            return get_arch('x86_64:wincc')
-    elif sys.platform == 'linux':
-        if platform.architecture()[0] == '64bit':
-            return get_arch('x86_64')
-
-
 def construct(buildfile, targets=()):
     """ Construct the given buildfile.
 
@@ -203,37 +165,6 @@ def disasm(data, march):
     disassembler.disasm(data, ostream)
 
 
-def c3toir(sources, includes, march, reporter=None):
-    """ Compile c3 sources to ir-code for the given architecture. """
-    logger = logging.getLogger('c3c')
-    march = get_arch(march)
-    if not reporter:  # pragma: no cover
-        reporter = DummyReportGenerator()
-
-    logger.debug('C3 compilation started')
-    reporter.heading(2, 'c3 compilation')
-    sources = [get_file(fn) for fn in sources]
-    includes = [get_file(fn) for fn in includes]
-    diag = DiagnosticsManager()
-    c3b = C3Builder(diag, march.info)
-
-    try:
-        _, ir_modules = c3b.build(sources, includes)
-        for ircode in ir_modules:
-            verify_module(ircode)
-    except CompilerError as ex:
-        diag.error(ex.msg, ex.loc)
-        diag.print_errors()
-        raise TaskError('Compile errors')
-
-    reporter.message('C3 compilation listings for {}'.format(sources))
-    for ir_module in ir_modules:
-        reporter.message('{} {}'.format(ir_module, ir_module.stats()))
-        reporter.dump_ir(ir_module)
-
-    return ir_modules
-
-
 OPT_LEVELS = ('0', '1', '2', 's')
 
 
@@ -272,6 +203,7 @@ def optimize(ir_module, level=0, reporter=None):
                   RemoveAddZeroPass(),
                   ConstantFolder(),
                   CommonSubexpressionEliminationPass(),
+                  TailCallOptimization(),
                   LoadAfterStorePass(),
                   DeleteUnusedInstructionsPass(),
                   CleanPass()] * 3
@@ -400,31 +332,28 @@ def cc(source: io.TextIOBase, march, coptions=None, opt_level=0,
     return ir_to_object([ir_module], march, debug=debug, reporter=reporter)
 
 
-def wasmcompile(source: io.TextIOBase, march, opt_level=2):
+def wasmcompile(source: io.TextIOBase, march, opt_level=2, reporter=None):
     """ Webassembly compile """
     march = get_arch(march)
+
+    if not reporter:  # pragma: no cover
+        reporter = DummyReportGenerator()
+
     wasm_module = read_wasm(source)
-    ir_module = wasm_to_ir(wasm_module)
+    ir_module = wasm_to_ir(
+        wasm_module, march.info.get_type_info('ptr'), reporter=reporter)
 
     # Optimize:
     optimize(ir_module, level=opt_level)
 
-    obj = ir_to_object([ir_module], march)
+    obj = ir_to_object([ir_module], march, reporter=reporter)
     return obj
 
 
-def llvm_to_ir(source):
-    """ Convert llvm assembly code into an IR-module """
-    llvm = LlvmIrFrontend()
-    ir_module = llvm.compile(source)
-    return ir_module
-
-
 def llc(source, march):
     """ Compile llvm assembly source into machine code """
     march = get_arch(march)
-    llvm = LlvmIrFrontend()
-    ir_module = llvm.compile(source)
+    ir_module = llvm_to_ir(source)
     return ir_to_object([ir_module], march)
 
 
@@ -454,15 +383,14 @@ def c3c(sources, includes, march, opt_level=0, reporter=None, debug=False,
     """
     reporter = get_reporter(reporter)
     march = get_arch(march)
-    ir_modules = \
-        c3toir(sources, includes, march, reporter=reporter)
+    ir_module = \
+        c3_to_ir(sources, includes, march, reporter=reporter)
 
-    for ircode in ir_modules:
-        optimize(ircode, level=opt_level, reporter=reporter)
+    optimize(ir_module, level=opt_level, reporter=reporter)
 
     opt_cg = 'size' if opt_level == 's' else 'speed'
     return ir_to_object(
-        ir_modules, march, debug=debug, reporter=reporter,
+        [ir_module], march, debug=debug, reporter=reporter,
         opt=opt_cg, outstream=outstream)
 
 
@@ -488,25 +416,6 @@ def pascal(sources, march, opt_level=0, reporter=None):
     return ir_to_object(ir_modules, march, reporter=reporter)
 
 
-def bf2ir(source, target):
-    """ Compile brainfuck source into ir code """
-    target = get_arch(target)
-    ircode = BrainFuckGenerator(target).generate(source)
-    return ircode
-
-
-def ws2ir(source):
-    """ Compile whitespace source """
-    WhitespaceGenerator().compile(source)
-
-
-def fortran_to_ir(source):
-    """ Translate fortran source into IR-code """
-    builder = FortranBuilder()
-    ir_modules = builder.build(source)
-    return ir_modules
-
-
 def bfcompile(source, target, reporter=None):
     """ Compile brainfuck source into binary format for the given target
 
@@ -530,7 +439,7 @@ def bfcompile(source, target, reporter=None):
         reporter = DummyReportGenerator()
     reporter.message('brainfuck compilation listings')
     target = get_arch(target)
-    ir_module = bf2ir(source, target)
+    ir_module = bf_to_ir(source, target)
     reporter.message(
         'Before optimization {} {}'.format(ir_module, ir_module.stats()))
     reporter.dump_ir(ir_module)
@@ -556,11 +465,6 @@ def fortrancompile(sources, target, reporter=DummyReportGenerator()):
     return ir_to_object(ir_modules, target, reporter=reporter)
 
 
-def llvmir2ir(f):
-    """ Parse llvm IR-code into a ppci ir-module """
-    return LlvmIrFrontend().compile(f)
-
-
 def objcopy(obj: ObjectFile, image_name: str, fmt: str, output_filename):
     """ Copy some parts of an object file to an output """
     fmts = ['bin', 'hex', 'elf', 'exe', 'ldb', 'uimage']
@@ -574,20 +478,19 @@ def objcopy(obj: ObjectFile, image_name: str, fmt: str, output_filename):
         with open(output_filename, 'wb') as output_file:
             output_file.write(image.data)
     elif fmt == "elf":
-        elf_file = ElfFile()
         with open(output_filename, 'wb') as output_file:
-            elf_file.save(output_file, obj)
+            write_elf(obj, output_file)
         status = os.stat(output_filename)
         os.chmod(output_filename, status.st_mode | stat.S_IEXEC)
     elif fmt == "hex":
         image = obj.get_image(image_name)
         hexfile = HexFile()
         hexfile.add_region(image.address, image.data)
-        with open(output_filename, 'w') as output_file:
+        with open(output_filename, 'wt', encoding='utf8') as output_file:
             hexfile.save(output_file)
     elif fmt == 'ldb':
         # TODO: fix this some other way to extract debug info
-        with open(output_filename, 'w') as output_file:
+        with open(output_filename, 'wt', encoding='utf8') as output_file:
             write_ldb(obj, output_file)
     elif fmt == 'uimage':
         image = obj.get_image(image_name)
