@@ -2,12 +2,6 @@
 
 The is the avr backend.
 
-.. autoclass:: ppci.arch.avr.AvrArch
-
-.. autoclass:: ppci.arch.avr.registers.AvrRegister
-
-.. autoclass:: ppci.arch.avr.registers.AvrWordRegister
-
 See also:
 
 https://gcc.gnu.org/wiki/avr-gcc
@@ -17,5 +11,6 @@ http://www.atmel.com/webdoc/avrassembler/avrassembler.wb_instruction_list.html
 """
 
 from .arch import AvrArch
+from .registers import AvrRegister, AvrWordRegister
 
-__all__ = ['AvrArch']
+__all__ = ['AvrArch', 'AvrRegister', 'AvrWordRegister']
