@@ -480,9 +480,11 @@ def pattern_jmp(context, tree):
 @isa.pattern('stm', 'CJMPI8(reg, reg)', size=10)
 def pattern_cjmp(context, tree, lhs, rhs):
     op, true_tgt, false_tgt = tree.value
-    opnames = {"<": Jl, ">": Jl, "==": Jz, "!=": Jne, ">=": Jge}
+    opnames = {
+        "<": Jl, ">": Jl, "==": Jz, "!=": Jne, ">=": Jge, '<=': Jge,
+    }
     op_ins = opnames[op]
-    if op in ['>']:
+    if op in ['>', '<=']:
         # Swap operands here!
         # This is really hairy code, but it should work!
         lhs, rhs = rhs, lhs
@@ -493,15 +495,15 @@ def pattern_cjmp(context, tree, lhs, rhs):
     context.emit(jmp_ins)
 
 
-@isa.pattern('reg', 'MOVI16(reg)', size=2, cycles=1, energy=1)
-@isa.pattern('reg', 'MOVU16(reg)', size=2, cycles=1, energy=1)
+@isa.pattern('stm', 'MOVI16(reg)', size=2, cycles=1, energy=1)
+@isa.pattern('stm', 'MOVU16(reg)', size=2, cycles=1, energy=1)
 def pattern_mov16(context, tree, c0):
     dst = tree.value
     context.emit(mov(c0, dst))
 
 
-@isa.pattern('reg', 'MOVI8(reg)', size=2)
-@isa.pattern('reg', 'MOVU8(reg)', size=2)
+@isa.pattern('stm', 'MOVI8(reg)', size=2)
+@isa.pattern('stm', 'MOVU8(reg)', size=2)
 def pattern_mov8(context, tree, c0):
     dst = tree.value
     context.emit(mov(c0, dst))
