@@ -17,10 +17,21 @@ from .registers import R9, R18, R19
 from .registers import R20, R21, R22, R23, R24, R25, R26, R27
 from ... import ir
 from ..registers import RegisterClass
+from ..stack import StackLocation
+from ..stack import FramePointerLocation 
 from ..data_instructions import data_isa
 from ...binutils.assembler import BaseAssembler
-from .instructions import dcd, Addi, Movr, Bl, Sw, Lw, Blr
-from .rvc_instructions import CSwsp, CLwsp, CJal, CJr, CJalr
+from .instructions import dcd, Addi, Movr, Bl, Sw, Lw, Blr, Lb, Sb
+from .rvc_instructions import CSwsp, CLwsp, CBl, CJr, CBlr, CMovr
+from .rvc_instructions import CLwsp, CSwsp, CAddi16sp, CAddi4spn
+
+def isinsrange(bits, val):
+        msb = 1<<(bits-1)
+        ll = -msb
+        if (val<=(msb-1) and (val >=ll)):
+            return True
+        else:    
+            return False
 
 
 class RiscvAssembler(BaseAssembler):
@@ -61,6 +72,7 @@ class RiscvArch(Architecture):
             self.isa = isa + data_isa
             self.store = Sw
             self.load = Lw
+        self.fp_location = FramePointerLocation.BOTTOM 
         self.isa.sectinst = Section
         self.isa.dbinst = DByte
         self.isa.dsinst = DZero
@@ -73,7 +85,7 @@ class RiscvArch(Architecture):
         # Allocatable registers:
         register_classes = [
             RegisterClass(
-                'reg', [ir.i8, ir.i16, ir.i32, ir.ptr, ir.u8, ir.u16, ir.u32],
+                'reg', [ir.i8, ir.i16, ir.i32, ir.ptr, ir.u8, ir.u16, ir.u32, ir.f32, ir.f64],
                 RiscvRegister,
                 [
                     R9, R10, R11, R12, R13, R14, R15, R16, R17, R18, R19, R20,
@@ -86,7 +98,8 @@ class RiscvArch(Architecture):
                 ir.i8: TypeInfo(1, 1), ir.u8: TypeInfo(1, 1),
                 ir.i16: TypeInfo(2, 2), ir.u16: TypeInfo(2, 2),
                 ir.i32: TypeInfo(4, 4), ir.u32: TypeInfo(4, 4),
-                'int': ir.i32, 'ptr': ir.u32,
+                ir.f32: TypeInfo(4, 4), ir.f64: TypeInfo(4, 4),
+                'int': ir.i32, 'ptr': ir.u32, ir.ptr: ir.u32,
             }, register_classes=register_classes)
 
         self.fp = FP
@@ -98,9 +111,9 @@ class RiscvArch(Architecture):
     def branch(self, reg, lab):
         if self.has_option('rvc'):
             if isinstance(lab, RiscvRegister):
-                return CJalr(lab, clobbers=self.caller_save)
+                return CBlr(reg, lab, 0, clobbers=self.caller_save)
             else:
-                return CJal(lab, clobbers=self.caller_save)
+                return CBl(reg, lab, clobbers=self.caller_save)
         else:
             if isinstance(lab, RiscvRegister):
                 return Blr(reg, lab, 0, clobbers=self.caller_save)
@@ -144,7 +157,17 @@ class RiscvArch(Architecture):
 
     def move(self, dst, src):
         """ Generate a move from src to dst """
-        return Movr(dst, src, ismove=True)
+        if self.has_option('rvc'):
+            return CMovr(dst, src, ismove=True)
+        else:
+            return Movr(dst, src, ismove=True)
+
+    def gen_riscv_memcpy(self, dst, src, tmp, size):
+        # Called before register allocation
+        # Major crappy memcpy, can be improved!
+        for idx in range(size):
+            yield Lb(tmp, idx, src)
+            yield Sb(tmp, idx, dst)
 
     def gen_call(self, frame, label, args, rv):
         """ Implement actual call and save / restore live registers """
@@ -180,6 +203,8 @@ class RiscvArch(Architecture):
             arg = arg2[1]
             if isinstance(arg_loc, Register):
                 yield self.move(arg, arg_loc)
+            elif isinstance(arg_loc, StackLocation):
+                pass
             else:  # pragma: no cover
                 raise NotImplementedError('Parameters in memory not impl')
 
@@ -198,12 +223,22 @@ class RiscvArch(Architecture):
             pass args in R12-R17
             return values in R10
         """
-        l = []
+        locations = []
         regs = [R12, R13, R14, R15, R16, R17]
+        offset = 0
         for a in arg_types:
-            r = regs.pop(0)
-            l.append(r)
-        return l
+            if a.is_blob:
+                r = StackLocation(offset, a.size)
+                offset += a.size
+            else:
+                if regs:
+                    r = regs.pop(0)
+                else:
+                    arg_size = self.info.get_size(a)
+                    r = StackLocation(offset, arg_size)
+                    offset += arg_size
+            locations.append(r)
+        return locations
 
     def determine_rv_location(self, ret_type):
         rv = R10
@@ -213,21 +248,57 @@ class RiscvArch(Architecture):
         """ Returns prologue instruction sequence """
         # Label indication function:
         yield Label(frame.name)
-        ssize = round_up(frame.stacksize) + 8
-        yield Sw(LR, -ssize + 4, SP)
-        yield Sw(FP, -ssize, SP)
-        yield Movr(FP, SP)  # Setup frame pointer
-        yield Addi(SP, SP, -ssize)  # Reserve stack space
+        ssize = round_up(frame.stacksize + 8)
+        if self.has_option('rvc') and isinsrange(10, -ssize):
+            yield CAddi16sp(-ssize)  # Reserve stack space
+        else:
+            yield Addi(SP, SP, -ssize)  # Reserve stack space            
+        
+        if self.has_option('rvc'):
+            yield CSwsp(LR, 4)
+            yield CSwsp(FP, 0)
+        else:
+            yield Sw(LR, 4, SP)
+            yield Sw(FP, 0, SP)
+        
+        if self.has_option('rvc'):
+            yield CAddi4spn(FP, 8)  # Setup frame pointer
+        else:
+            yield Addi(FP, SP, 8)  # Setup frame pointer        
+        #yield Addi(FP, SP, 8)  # Setup frame pointer        
+        
+        rsize = 0
+        for register in self.callee_save:
+            if frame.is_used(register):
+                rsize += 4 
+        rsize = round_up(rsize)         
+        if self.has_option('rvc') and isinsrange(10, rsize):
+            yield CAddi16sp(-rsize)  # Reserve stack space
+        else:
+            yield Addi(SP, SP, -rsize)  # Reserve stack space
         i = 0
         for register in self.callee_save:
             if frame.is_used(register):
                 i -= 4
-                yield Sw(register, i, SP)
-        yield Addi(SP, SP, i)
+                if self.has_option('rvc'):
+                    yield CSwsp(register,i + rsize)
+                else:
+                    yield Sw(register, i + rsize, SP)
+        
+
+        # Allocate space for outgoing calls:
+        extras = max(frame.out_calls) if frame.out_calls else 0
+        if extras:
+            ssize = round_up(extras)
+            if self.has_option('rvc') and isinsrange(10, ssize):
+                yield CAddi16sp(-ssize)  # Reserve stack space
+            else:
+                yield Addi(SP, SP, -ssize)  # Reserve stack space            
 
     def litpool(self, frame):
         """ Generate instruction for the current literals """
-        # Align at 4 bytes
+        yield Section('data')
+        # Align at 4 byte
         if frame.constants:
             yield Align(4)
 
@@ -235,7 +306,7 @@ class RiscvArch(Architecture):
         while frame.constants:
             label, value = frame.constants.pop(0)
             yield Label(label)
-            if isinstance(value, int) or isinstance(value, str):
+            if isinstance(value, (int, str)):
                 yield dcd(value)
             elif isinstance(value, bytes):
                 for byte in value:
@@ -243,6 +314,8 @@ class RiscvArch(Architecture):
                 yield Align(4)  # Align at 4 bytes
             else:  # pragma: no cover
                 raise NotImplementedError('Constant of type {}'.format(value))
+        
+        yield Section('code')
 
     def between_blocks(self, frame):
         for ins in self.litpool(frame):
@@ -252,23 +325,53 @@ class RiscvArch(Architecture):
         """ Return epilogue sequence for a frame. Adjust frame pointer
             and add constant pool
         """
+        # Free space for outgoing calls:
+        extras = max(frame.out_calls) if frame.out_calls else 0
+        if extras:
+            ssize = round_up(extras)            
+            if self.has_option('rvc') and isinsrange(10, ssize):
+                yield CAddi16sp(ssize)  # Reserve stack space
+            else:
+                yield Addi(SP, SP, ssize)  # Reserve stack space            
         # Callee saved registers:
-        i = 0
-        for register in reversed(self.callee_save):
+        rsize = 0
+        for register in self.callee_save:
             if frame.is_used(register):
-                yield Lw(register, i, SP)
-                i += 4
-        yield Addi(SP, SP, i)
-        ssize = round_up(frame.stacksize) + 8
-        yield Addi(SP, SP, ssize)
-        yield Lw(FP, -ssize, SP)
-        yield Lw(LR, -ssize + 4, SP)
-
+                rsize += 4 
+        rsize = round_up(rsize)
+        
+        i = 0
+        for register in self.callee_save:
+            if frame.is_used(register):
+                i -= 4
+                if self.has_option('rvc'):
+                    yield CLwsp(register, i + rsize)
+                else:
+                    yield Lw(register, i + rsize, SP)              
+               
+        if self.has_option('rvc') and isinsrange(10, rsize):
+            yield CAddi16sp(rsize)  # Reserve stack space
+        else:
+            yield Addi(SP, SP, rsize)  # Reserve stack space              
+        
+        if self.has_option('rvc'):
+            yield CLwsp(LR, 4)
+            yield CLwsp(FP, 0)
+        else:
+            yield Lw(LR, 4, SP)
+            yield Lw(FP, 0, SP)
+        
+        ssize = round_up(frame.stacksize + 8) 
+        if self.has_option('rvc') and isinsrange(10, ssize):
+            yield CAddi16sp(ssize)  # Free stack space
+        else:
+            yield Addi(SP, SP, ssize)  # Free stack space
+            
         # Return
         if self.has_option('rvc'):
-            yield (CJr(LR))
+            yield CJr(LR)
         else:
-            yield (Blr(R0, LR, 0))
+            yield Blr(R0, LR, 0)
 
         # Add final literal pool:
         for instruction in self.litpool(frame):
@@ -277,4 +380,4 @@ class RiscvArch(Architecture):
 
 
 def round_up(s):
-    return s + (4 - s % 4)
+    return s + (16 - s % 16)
