@@ -3,12 +3,13 @@
 from ..isa import Isa
 from ..encoding import Instruction, Syntax, Operand
 from .registers import RiscvRegister
-from .tokens import RiscvcToken
+from .tokens import RiscvToken, RiscvcToken
 from .rvc_relocations import BcImm11Relocation, BcImm8Relocation
+from .rvc_relocations import CBImm11Relocation, CBlImm11Relocation, CRel
 from ..generic_instructions import ArtificialInstruction
 from .instructions import Andr, Orr, Xorr, Subr, Addi, Slli, Srli
-from .instructions import Lw, Sw, Blt, Bgt, Bge, B, Beq, Bne, Ble
-
+from .instructions import Lw, Sw, Blt, Bgt, Bge, B, Beq, Bne, Ble, Blr, Bgtu, Bltu, Bgeu, Bleu
+import logging
 
 class RegisterSet(set):
     def __repr__(self):
@@ -18,14 +19,21 @@ class RegisterSet(set):
 
 rvcisa = Isa()
 
+rvcisa.register_relocation(CRel)
 rvcisa.register_relocation(BcImm11Relocation)
 rvcisa.register_relocation(BcImm8Relocation)
+rvcisa.register_relocation(CBImm11Relocation)
+rvcisa.register_relocation(CBlImm11Relocation)
 
 
 class RiscvcInstruction(Instruction):
     tokens = [RiscvcToken]
     isa = rvcisa
 
+class RiscvInstruction(Instruction):
+    tokens = [RiscvToken]
+    isa = rvcisa 
+
 
 class PseudoRiscvInstruction(ArtificialInstruction):
     """ These instruction is used to switch between RV and RVC-encoding """
@@ -132,6 +140,33 @@ class CEbreak(RiscvcInstruction):
         tokens[0][0:16] = 0b1001000000000010
         return tokens[0].encode()
 
+class CMovr(RiscvcInstruction):    
+    rd = Operand('rd', RiscvRegister, write=True)
+    rm = Operand('rm', RiscvRegister, read=True)
+    syntax = Syntax(['c', '.','mv', ' ', rd, ',', ' ', rm])
+    
+    def encode(self):
+        tokens = self.get_tokens()
+        tokens[0][0:2] = 0b10
+        tokens[0][2:7] = self.rm.num
+        tokens[0][7:12] = self.rd.num
+        tokens[0][12:16] = 0b1000
+        return tokens[0].encode()
+
+        
+class CBl(RiscvInstruction):
+    target = Operand('target', str)
+    rd = Operand('rd', RiscvRegister, write=True)
+    syntax = Syntax(['jal', ' ', rd, ',', ' ', target])
+
+    def encode(self):
+        tokens = self.get_tokens()
+        tokens[0][0:7] = 0b1101111
+        tokens[0][7:12] = self.rd.num
+        return tokens[0].encode()
+
+    def relocations(self):
+        return [CBlImm11Relocation(self.target)] 
 
 class CJal(RiscvcInstruction):
     target = Operand('target', str)
@@ -147,6 +182,20 @@ class CJal(RiscvcInstruction):
         return [BcImm11Relocation(self.target)]
 
 
+class CB(RiscvInstruction):
+    target = Operand('target', str)
+    syntax = Syntax(['j', ' ', target])
+
+    def encode(self):
+        tokens = self.get_tokens()
+        tokens[0][0:7] = 0b1101111
+        tokens[0][7:12] = 0
+        return tokens[0].encode()
+
+    def relocations(self):
+        return [CBImm11Relocation(self.target)] 
+        
+        
 class CJ(RiscvcInstruction):
     target = Operand('target', str)
     syntax = Syntax(['c', '.', 'j', ' ', target])
@@ -172,6 +221,18 @@ class CJr(RiscvcInstruction):
         tokens[0][12:16] = 0b1000
         return tokens[0].encode()
 
+class CBlr(PseudoRiscvInstruction):
+    rd = Operand('rd', RiscvRegister, write=True)
+    rs1 = Operand('rs1', RiscvRegister, read=True)
+    offset = Operand('offset', int)
+    syntax = Syntax(['jalr', ' ', rd, ',', rs1, ',', ' ', offset]) 
+    
+    def render(self):
+        if self.rd.num == 1 and not self.offset:                        
+            yield CJalr(self.rs1)
+        else:
+            yield Blr(self.rd, self.rs1, self.offset)
+    
 
 class CJalr(RiscvcInstruction):
     rs1 = Operand('rs1', RiscvRegister, read=True)
@@ -258,8 +319,8 @@ class CSw(RiscvcInstruction):
 class CLwsp(RiscvcInstruction):
     rd = Operand('rd', RiscvRegister, write=True)
     offset = Operand('offset', int)
-    rs1 = Operand('rs1', RiscvRegister, read=True)
-    syntax = Syntax(['c', '.', 'lwsp', ' ', rd, ',', offset, '(', rs1, ')'])
+    #rs1 = Operand('rs1', RiscvRegister, read=True)
+    syntax = Syntax(['c', '.', 'lwsp', ' ', rd, ',', offset, '(', 'x2', ')'])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -271,12 +332,43 @@ class CLwsp(RiscvcInstruction):
         tokens[0][13:16] = 0b010
         return tokens[0].encode()
 
+class CAddi4spn(RiscvcInstruction): 
+    rd = Operand('rd', RiscvRegister, write=True)
+    imm = Operand('imm', int)
+    syntax = Syntax(['c', '.', 'addi4spn', ' ', rd,  ' ', imm])
+
+    def encode(self):
+        tokens = self.get_tokens()
+        tokens[0][0:2] = 0b00
+        tokens[0][2:5] = self.rd.num - 8
+        tokens[0][5:6] = self.imm >> 3 & 1        
+        tokens[0][6:7] = self.imm >> 2 & 1 
+        tokens[0][7:11] = self.imm >> 6 & 0xf
+        tokens[0][11:13] = self.imm >> 4 & 0x3
+        tokens[0][13:16] = 0b000
+        return tokens[0].encode()
+        
+class CAddi16sp(RiscvcInstruction):    
+    imm = Operand('imm', int)
+    syntax = Syntax(['c', '.', 'addi16sp', ' ', imm])
+
+    def encode(self):
+        tokens = self.get_tokens()
+        tokens[0][0:2] = 0b01
+        tokens[0][2:3] = self.imm >> 5 & 1
+        tokens[0][3:5] = self.imm >> 7 & 3        
+        tokens[0][5:6] = self.imm >> 6 & 1
+        tokens[0][6:7] = self.imm >> 4 & 1
+        tokens[0][7:12] = 2
+        tokens[0][12:13] = self.imm >> 9 & 1
+        tokens[0][13:16] = 0b011
+        return tokens[0].encode()
 
 class CSwsp(RiscvcInstruction):
     rs2 = Operand('rs2', RiscvRegister, read=True)
     offset = Operand('offset', int)
-    rs1 = Operand('rs1', RiscvRegister, read=True)
-    syntax = Syntax(['c', '.', 'swsp', ' ', rs2, ',', offset, '(', rs1, ')'])
+    #rs1 = Operand('rs1', RiscvRegister, read=True)
+    syntax = Syntax(['c', '.', 'swsp', ' ', rs2, ',', offset, '(', 'x2', ')'])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -582,11 +674,12 @@ def pattern_stri32_const(context, tree, c0):
     context.emit(Srliv(d, c0, c1))
     return d
 
-
-@rvcisa.pattern('reg', 'LDRI32(reg)', size=1)
+@rvcisa.pattern('reg', 'LDRU32(mem)', size=1)
+@rvcisa.pattern('reg', 'LDRI32(mem)', size=1)
 def pattern_ldri32(context, tree, c0):
     d = context.new_reg(RiscvRegister)
-    context.emit(Lwv(d, 0, c0))
+    base_reg, offset = c0 
+    context.emit(Lwv(d, offset, base_reg))
     return d
 
 
@@ -598,10 +691,11 @@ def pattern_ldri32_addi32(context, tree, c0):
     context.emit(Lwv(d, c1, c0))
     return d
 
-
-@rvcisa.pattern('stm', 'STRI32(reg, reg)', size=1)
+@rvcisa.pattern('stm', 'STRU32(mem, reg)', size=1)
+@rvcisa.pattern('stm', 'STRI32(mem, reg)', size=1)
 def pattern_stri32(self, tree, c0, c1):
-    self.emit(Swv(c1, 0, c0))
+    base_reg, offset = c0 
+    self.emit(Swv(c1, offset, base_reg))
 
 
 @rvcisa.pattern(
@@ -615,11 +709,115 @@ def pattern_stri32_addi32(context, tree, c0, c1):
 
 
 @rvcisa.pattern('stm', 'CJMPI32(reg, reg)', size=2)
+@rvcisa.pattern('stm', 'CJMPI16(reg, reg)', size=2)
 @rvcisa.pattern('stm', 'CJMPI8(reg, reg)', size=2)
 def pattern_cjmp(context, tree, c0, c1):
     op, yes_label, no_label = tree.value
     opnames = {"<": Blt, ">": Bgt, "==": Beq, "!=": Bne, ">=": Bge, "<=": Ble}
     Bop = opnames[op]
-    jmp_ins = B(no_label.name, jumps=[no_label])
+    jmp_ins = CB(no_label.name, jumps=[no_label])
     context.emit(Bop(c0, c1, yes_label.name, jumps=[yes_label, jmp_ins]))
     context.emit(jmp_ins)
+
+@rvcisa.pattern('stm', 'CJMPU8(reg, reg)', size=2)
+@rvcisa.pattern('stm', 'CJMPU16(reg, reg)', size=2)
+@rvcisa.pattern('stm', 'CJMPU32(reg, reg)', size=2)
+def pattern_cjmpu(context, tree, c0, c1):
+    op, yes_label, no_label = tree.value
+    opnames = {
+        "<": Bltu, ">": Bgtu, "==": Beq, "!=": Bne, ">=": Bgeu, "<=": Bleu
+    }
+    Bop = opnames[op]
+    jmp_ins = CB(no_label.name, jumps=[no_label])
+    context.emit(Bop(c0, c1, yes_label.name, jumps=[yes_label, jmp_ins]))
+    context.emit(jmp_ins) 
+    
+    
+@rvcisa.pattern('stm', 'JMP', size=2)
+def pattern_jmp(context, tree):
+    tgt = tree.value
+    context.emit(CB(tgt.name, jumps=[tgt]))    
+    
+@rvcisa.postlink
+def opt(cls, dst):
+    logger = logging.getLogger('linker')
+    lst = dst.arch.isa.relocation_map['c_base'].l
+    s = ", ".join(format(x, '08x') for x in lst)
+    logger.debug('Relocation list: %s\n' %s)
+    lst.append(-1)
+    
+    def countdiff(adrdst, adrofs):
+        indofs = 0
+        while lst[indofs]>=0 and adrofs > lst[indofs]:
+            indofs += 1
+        inddst = 0
+        while lst[inddst]>=0 and adrdst > lst[inddst]:
+            inddst += 1
+        return inddst - indofs
+    
+    def getsection(adr, image):
+        if adr>=imageadr and adr<=imageadr+imagesize:
+            for section in image.sections:
+                if adr>=sectionadr[section.name] and \
+                adr<=sectionadr[section.name]+sectionsize[section.name]:
+                    return section
+        else:
+            return False
+    
+    for image in dst.images:  
+        sectionnames = []
+        sectionadr = {}
+        sectionsize = {}
+        imagesize = image.size
+        imageadr = image.address
+        for s in image.sections:
+            sectionnames.append(s.name)
+            sectionadr[s.name] = s.address
+            sectionsize[s.name] = s.size
+        symbols = []
+        relocs = []
+        
+        for s in dst.symbols:
+            if s.section in sectionnames:
+                section = dst.get_section(s.section)
+                symbols.append((s, s.value + section.address))
+        symbols.sort(key=lambda x: x[1])
+        
+        for r in dst.relocations:
+            if r.section in sectionnames:
+                 section = dst.get_section(r.section)
+                 relocs.append((r, r.offset + section.address))
+        relocs.sort(key=lambda x: x[1])
+           
+        
+        for se in image.sections:
+            delta = countdiff(se.address, image.address)
+            logger.debug('RVC-sectororchanging %s at %08x with -%08x to %08x' %(se.name, se.address, 2*delta, se.address-2*delta))
+            se.address -= 2*delta
+        
+        for s in symbols:
+            sym = s[0]
+            delta = countdiff(s[1], sectionadr[sym.section])
+            logger.debug('RVC-symbolchanging %s at %08x with -%08x to %08x' %(sym.name, sym.value, 2*delta, sym.value-2*delta))
+            sym.value -= 2*delta
+            
+        
+        for r in relocs:
+            reloc = r[0]
+            delta = countdiff(r[1], sectionadr[reloc.section])
+            logger.debug('RVC-relocationchanging %s at %08x with -%08x to %08x' %(reloc.symbol_name, r[1], 2*delta, r[1]-2*delta))
+            reloc.offset -= delta*2
+        
+        lst2 = lst[:-1]
+        for (i,absadr) in enumerate(lst2):
+            section = getsection(absadr, image)
+            if section:
+                adr = absadr - section.address + 2 - i*2
+                section.data.pop(adr)
+                section.data.pop(adr)
+    
+    
+    cls.do_relocations(dst, opt=True)
+    dst.arch.isa.relocation_map['c_base'].l = []
+    
+
