@@ -1,20 +1,21 @@
 """ C front end. """
 
 from .context import CContext
-from .builder import CBuilder, create_ast
+from .builder import CBuilder, create_ast, parse_text, parse_type
 from .lexer import CLexer
 from .parser import CParser
 from .semantics import CSemantics
 from .synthesize import CSynthesizer
 from .preprocessor import CPreProcessor
-from .utils import CAstPrinter
-from .printer import CPrinter
+from .utils import CAstPrinter, print_ast
+from .printer import CPrinter, render_ast
 from .options import COptions
 from .api import preprocess, c_to_ir
 
 
 __all__ = [
-    'create_ast', 'preprocess', 'c_to_ir',
+    'create_ast', 'preprocess', 'c_to_ir', 'print_ast', 'parse_text',
+    'render_ast', 'parse_type',
     'CBuilder', 'CContext', 'CLexer', 'COptions', 'CPreProcessor', 'CParser',
     'CAstPrinter', 'CSemantics', 'CSynthesizer', 'CPrinter'
     ]
