@@ -3,41 +3,9 @@
 import logging
 import io
 
-from ..common import Token, SourceLocation
-from ...common import CompilerError
-
-
-class CToken(Token):
-    """ C token (including optional preceeding spaces) """
-    def __init__(self, typ, val, space, first, loc):
-        super().__init__(typ, val, loc)
-        self.space = space
-        self.first = first
-
-    def __repr__(self):
-        return 'CToken({}, {}, {}, "{}", {})'.format(
-            self.typ, self.val, self.first, self.space, self.loc)
-
-    def __str__(self):
-        return self.space + self.val
-
-    def copy(self, space=None, first=None):
-        """ Return a new token which is a mildly modified copy """
-        if space is None:
-            space = self.space
-        if first is None:
-            first = self.first
-        return CToken(self.typ, self.val, space, first, self.loc)
-
-
-class Char:
-    """ Represents a single character with a location """
-    def __init__(self, char, loc):
-        self.char = char
-        self.loc = loc
-
-    def __repr__(self):
-        return "CHAR '{}' at {}".format(self.char, self.loc)
+from ..common import SourceLocation
+from .token import CToken
+from ..tools.handlexer import HandLexerBase, Char
 
 
 def create_characters(f, filename):
@@ -103,97 +71,6 @@ def lex_text(text, coptions):
     return list(lexer.lex_text(text))
 
 
-class HandLexerBase:
-    """ Base class for handwritten lexers based on an idea of Rob Pike.
-
-    See also:
-    http://eli.thegreenplace.net/2012/08/09/
-    using-sub-generators-for-lexical-scanning-in-python/
-
-    And:
-    https://www.youtube.com/watch?v=HxaD_trXwRE
-    """
-    def __init__(self):
-        self.token_buffer = []
-        self.pushed_back = []
-        self.current_text = []
-
-    def tokenize(self, characters, start_state):
-        """ Return a sequence of tokens """
-        self.characters = characters
-        state = start_state
-        while state:
-            while self.token_buffer:
-                yield self.token_buffer.pop(0)
-            state = state()
-
-    def next_char(self) -> Char:
-        if self.pushed_back:
-            char = self.pushed_back.pop(0)
-        else:
-            char = next(self.characters, None)
-
-        if char:
-            self.current_text.append(char)
-
-        return char
-
-    def backup_char(self, char: Char):
-        """ go back one item """
-        if char:
-            self.current_text.pop(-1)
-            self.pushed_back.insert(0, char)
-
-    def emit(self, typ):
-        """ Emit the current text under scope as a token """
-        val = ''.join(c.char for c in self.current_text)
-        loc = self.current_text[0].loc
-        token = Token(typ, val, loc)
-        self.token_buffer.append(token)
-        self.current_text.clear()
-
-    def ignore(self):
-        """ Ignore text under cursor """
-        self.current_text.clear()
-
-    def accept(self, valid):
-        """ Accept a single character if it is in the valid set """
-        char = self.next_char()
-        if char and char.char in valid:
-            return True
-        else:
-            self.backup_char(char)
-            return False
-
-    def accept_run(self, valid):
-        while self.accept(valid):
-            pass
-
-    def accept_sequence(self, sequence):
-        """ Munch the exact given sequence of characters """
-        chars = []
-        for valid in sequence:
-            char = self.next_char()
-            chars.append(char)
-            if char and char.char in valid:
-                continue
-            else:
-                # Retreat! Pull back! We are wrong!
-                for char in reversed(chars):
-                    self.backup_char(char)
-                return False
-        return True
-
-    def error(self, message):
-        char = self.next_char()
-        loc = char.loc
-        raise CompilerError(message, loc)
-
-    def expect(self, valid):
-        if not self.accept(valid):
-            self.error("Expected {}".format(', '.join(valid)))
-
-
 class CLexer(HandLexerBase):
     """ Lexer used for the preprocessor """
     logger = logging.getLogger('clexer')
@@ -393,6 +270,9 @@ class CLexer(HandLexerBase):
         elif r.char == '.':
             if self.accept_sequence(['.', '.']):
                 self.emit('...')
+            elif self.accept(self.numbers):
+                # We got .[0-9]
+                return self.lex_float
             else:
                 self.emit('.')
             return self.lex_c
@@ -425,14 +305,25 @@ class CLexer(HandLexerBase):
 
         # Accept a series of number characters:
         self.accept_run(number_chars)
+        if self.accept('.'):
+            return self.lex_float()
+        else:
+            # Accept some suffixes:
+            self.accept('LlUu')
+            self.accept('LlUu')
+            # TODO: handle suffixes better
+            self.accept('LlUu')
 
-        # Accept some suffixes:
-        self.accept('LlUu')
-        self.accept('LlUu')
-        # TODO: handle suffixes better
-        self.accept('LlUu')
+            # self.accept('
+            self.emit('NUMBER')
+            return self.lex_c
+
+    def lex_float(self):
+        self.accept_run(self.numbers)
+        if self.accept('eEpP'):
+            self.accept('+-')
+            self.accept_run(self.numbers)
 
-        # self.accept('
         self.emit('NUMBER')
         return self.lex_c
 
@@ -457,16 +348,16 @@ class CLexer(HandLexerBase):
                     # self.emit('WS')
                     break
             else:
-                self.next_char()
+                self.next_char(eof=False)
         return self.lex_c
 
     def lex_string(self):
         """ Scan for a complete string """
-        c = self.next_char()
-        while c and c.char != '"':
+        c = self.next_char(eof=False)
+        while c.char != '"':
             if c.char == '\\':
                 self._handle_escape_character()
-            c = self.next_char()
+            c = self.next_char(eof=False)
         self.emit('STRING')
         return self.lex_c
 
@@ -476,7 +367,7 @@ class CLexer(HandLexerBase):
             self._handle_escape_character()
         else:
             # Normal char:
-            self.next_char()
+            self.next_char(eof=False)
 
         self.expect("'")
 
