@@ -10,7 +10,7 @@ The following parts can be distinguished:
 
 import logging
 from ..tools.recursivedescent import RecursiveDescentParser
-from . import nodes
+from .nodes import nodes
 
 
 class CParser(RecursiveDescentParser):
@@ -36,9 +36,8 @@ class CParser(RecursiveDescentParser):
     LEFT_ASSOCIATIVE = 'left-associative'
     RIGHT_ASSOCIATIVE = 'right-associative'
 
-    def __init__(self, context, semantics):
+    def __init__(self, coptions, semantics):
         super().__init__()
-        self.context = context
         self.semantics = semantics
         self.type_qualifiers = {'volatile', 'const'}
         self.storage_classes = {
@@ -52,10 +51,14 @@ class CParser(RecursiveDescentParser):
             'else', 'if', 'while', 'do', 'for', 'return', 'goto',
             'switch', 'case', 'default', 'break', 'continue',
             'sizeof', 'struct', 'union', 'enum',
-            '__builtin_va_arg', '__builtin_va_start'}
+            '__builtin_va_arg', '__builtin_va_start',
+            '__builtin_offsetof'}
+
+        # gcc extensions:
+        self.keywords.add('__attribute__')
 
         # Some additions from C99:
-        if self.context.coptions['std'] == 'c99':
+        if coptions['std'] == 'c99':
             self.type_qualifiers.add('restrict')
             self.keywords.add('inline')
 
@@ -80,7 +83,7 @@ class CParser(RecursiveDescentParser):
             '^=': (self.RIGHT_ASSOCIATIVE, 10),
             # '++': (LEFT_ASSOCIATIVE, 50),
             # '--': (LEFT_ASSOCIATIVE, 50),
-            '?': (self.LEFT_ASSOCIATIVE, 17),
+            '?': (self.RIGHT_ASSOCIATIVE, 17),
             '||': (self.LEFT_ASSOCIATIVE, 20),
             '&&': (self.LEFT_ASSOCIATIVE, 30),
             '|': (self.LEFT_ASSOCIATIVE, 40),
@@ -161,9 +164,10 @@ class CParser(RecursiveDescentParser):
         type_specifiers = []
         type_qualifiers = set()
         typ = None
+        attributes = []
 
         while True:
-            if self.peak == 'TYPE-ID':
+            if self.peek == 'TYPE-ID':
                 # We got a typedef type!
                 type_name = self.consume()
                 if typ:
@@ -171,24 +175,24 @@ class CParser(RecursiveDescentParser):
                 else:
                     typ = self.semantics.on_typename(
                         type_name.val, type_name.loc)
-            elif self.peak in self.type_specifiers:
+            elif self.peek in self.type_specifiers:
                 type_specifier = self.consume(self.type_specifiers)
                 location = type_specifier.loc
                 if typ:
                     self.error('Type already determined', type_specifier)
                 else:
                     type_specifiers.append(type_specifier.val)
-            elif self.peak == 'enum':
+            elif self.peek == 'enum':
                 if typ:
                     self.error('Unexpected enum')
                 else:
                     typ = self.parse_enum()
-            elif self.peak in ['struct', 'union']:
+            elif self.peek in ['struct', 'union']:
                 if typ:
                     self.error('Type already determined')
                 else:
                     typ = self.parse_struct_or_union()
-            elif self.peak in self.storage_classes:
+            elif self.peek in self.storage_classes:
                 klass = self.consume(self.storage_classes)
                 if not allow_storage_class:
                     self.error('Unexpected storage class', klass.loc)
@@ -196,16 +200,19 @@ class CParser(RecursiveDescentParser):
                     self.error('Multiple storage classes', klass.loc)
                 else:
                     storage_class = klass.val
-            elif self.peak in self.type_qualifiers:
+            elif self.peek in self.type_qualifiers:
                 type_qualifier = self.consume(self.type_qualifiers)
                 if type_qualifier.val in type_qualifiers:
                     self.error('Double type qualifier', type_qualifier.loc)
                 else:
                     type_qualifiers.add(type_qualifier.val)
-            elif self.peak in ['inline']:
+            elif self.peek in ['inline']:
                 self.consume('inline')
                 # inline is a compiler hint. For now, ignore this hint :)
                 self.logger.debug('Ignoring inline for now')
+            elif self.peek in ['__attribute__']:
+                attributes = self.parse_attributes()
+                self.logger.error('Ignoring %s', attributes)
             else:
                 break
 
@@ -228,83 +235,126 @@ class CParser(RecursiveDescentParser):
         keyword = self.consume({'struct', 'union'})
 
         # We might have an optional tag:
-        if self.peak == 'ID':
+        if self.peek == 'ID':
             tag = self.consume('ID').val
-        elif self.peak == '{':
+        elif self.peek == '{':
             tag = None
         else:
-            print(self.typedefs)
+            # print(self.typedefs)
             self.error(
-                'Expected tag name or "{{", but got {}'.format(self.peak),
+                'Expected tag name or "{{", but got {}'.format(self.peek),
                 keyword.loc)
 
-        if self.peak == '{':
-            # We have a struct declarations:
-            self.consume('{')
-            fields = []
-            while self.peak != '}':
-                ds = self.parse_decl_specifiers(allow_storage_class=False)
-                while True:
-                    type_modifiers, name = self.parse_type_modifiers(
-                        abstract=False)
-                    # Handle optional struct field size:
-                    if self.peak == ':':
-                        self.consume(':')
-                        bitsize = self.parse_constant_expression()
-                        # TODO: move this somewhere else?
-                    else:
-                        bitsize = None
-                    field = (
-                        ds.storage_class, ds.typ, name.val, type_modifiers,
-                        bitsize, name.loc)
-                    fields.append(field)
-                    if self.has_consumed(','):
-                        continue
-                    else:
-                        break
-                self.consume(';')
-            self.consume('}')
+        if self.peek == '{':
+            fields = self.parse_struct_fields()
         else:
             fields = None
 
         return self.semantics.on_struct_or_union(
             keyword.val, tag, fields, keyword.loc)
 
+    def parse_struct_fields(self):
+        """ Parse struct or union fields """
+        # We have a struct declarations:
+        self.consume('{')
+        fields = []
+        while self.peek != '}':
+            ds = self.parse_decl_specifiers(allow_storage_class=False)
+            while True:
+                type_modifiers, name = self.parse_type_modifiers(
+                    abstract=False)
+                # Handle optional struct field size:
+                if self.peek == ':':
+                    self.consume(':')
+                    bitsize = self.parse_constant_expression()
+                    # TODO: move this somewhere else?
+                else:
+                    bitsize = None
+                field = self.semantics.on_field_def(
+                    ds.storage_class, ds.typ, name.val, type_modifiers,
+                    bitsize, name.loc)
+                fields.append(field)
+                if self.has_consumed(','):
+                    continue
+                else:
+                    break
+            self.consume(';')
+        self.consume('}')
+        return fields
+
     def parse_enum(self):
         """ Parse an enum definition """
         keyword = self.consume('enum')
 
         # We might have an optional tag:
-        if self.peak == 'ID':
+        if self.peek == 'ID':
             tag = self.consume('ID').val
-        elif self.peak == '{':
+        elif self.peek == '{':
             tag = None
         else:
             self.error(
                 'Expected tag name or enum declaration, but got {}'.format(
-                    self.peak),
+                    self.peek),
                 keyword.loc)
 
         ctyp = self.semantics.on_enum(tag, keyword.loc)
 
         # If we have a body, either after tag or directly, parse it:
-        if self.peak == '{':
+        if self.peek == '{':
             self.consume('{')
             self.semantics.enter_enum_values(ctyp, keyword.loc)
-            while self.peak != '}':
+            constants = []
+            while self.peek != '}':
                 name = self.consume('ID')
                 if self.has_consumed('='):
                     value = self.parse_constant_expression()
                 else:
                     value = None
-                self.semantics.on_enum_value(ctyp, name.val, value, name.loc)
+                constant = self.semantics.on_enum_value(
+                    ctyp, name.val, value, name.loc)
+                constants.append(constant)
                 if not self.has_consumed(','):
                     break
             self.consume('}')
-            self.semantics.exit_enum_values(ctyp, keyword.loc)
+            self.semantics.exit_enum_values(ctyp, constants, keyword.loc)
 
         return ctyp
 
+    def parse_attributes(self):
+        """ Parse some attributes.
+
+        Examples are:
+
+        __attribute__((noreturn))
+        """
+        attributes = []
+        while True:
+            if self.peek == '__attribute__':
+                attribute = self.parse_gnu_attribute()
+            else:
+                break
+            attributes.append(attribute)
+        return attributes
+
+    def parse_gnu_attribute(self):
+        """ Parse a gnu attribute like __attribute__((noreturn)) """
+        self.consume('__attribute__')
+        self.consume('(')
+        self.consume('(')
+        gnu_attribute = {}
+        if self.peek != ')':
+            while True:
+                name = self.consume('ID')
+                # TODO: how use this?
+                gnu_attribute[name.val] = 1
+                if self.has_consumed(','):
+                    continue
+                else:
+                    break
+        self.consume(')')
+        self.consume(')')
+        return gnu_attribute
+
     def parse_decl_group(self, ds):
         """ Parse the rest after the first declaration spec.
 
@@ -321,7 +371,7 @@ class CParser(RecursiveDescentParser):
                 d = self.parse_declarator()
                 declarations.append(self.parse_typedef(ds, d))
             self.consume(';')
-        elif self.peak == '{':
+        elif self.peek == '{':
             # if function, parse implementation.
             # func_def = None
             declarations.append(self.parse_function_declaration(ds, d))
@@ -381,18 +431,18 @@ class CParser(RecursiveDescentParser):
 
     def parse_variable_initializer(self):
         """ Parse the C-style array or struct initializer stuff """
-        if self.peak == '{':
+        if self.peek == '{':
             location = self.consume('{').loc
             values = []
             values.append(self.parse_variable_initializer())
             while self.has_consumed(','):
-                if self.peak == '}':
+                if self.peek == '}':
                     break
                 values.append(self.parse_variable_initializer())
             self.consume('}')
             initializer = self.semantics.on_initializer_list(values, location)
         else:
-            if self.peak == '[' and self.options['std'] == 'c99':
+            if self.peek == '[' and self.options['std'] == 'c99':
                 # Parse designators like: int a[10] = {[1]=5, [4]=3, 4}
                 self.consume('[')
                 index = self.parse_constant_expression()
@@ -410,9 +460,9 @@ class CParser(RecursiveDescentParser):
         # TODO: allow K&R style arguments
         arguments = []
         # self.consume('(')
-        if self.peak != ')':
+        if self.peek != ')':
             while True:
-                if self.peak == '...':
+                if self.peek == '...':
                     # Variable arguments found!
                     self.consume('...')
                     arguments.append('...')
@@ -442,7 +492,7 @@ class CParser(RecursiveDescentParser):
         # Handle the pointer:
         while self.has_consumed('*'):
             type_qualifiers = set()
-            while self.peak in self.type_qualifiers:
+            while self.peek in self.type_qualifiers:
                 type_qualifier = self.consume(self.type_qualifiers)
                 if type_qualifier.val in type_qualifiers:
                     self.error('Duplicate type qualifier', type_qualifier)
@@ -453,9 +503,9 @@ class CParser(RecursiveDescentParser):
         # First parse some id, or something else
         middle_modifiers = []
         last_modifiers = []  # TODO: maybe move id detection in seperate part?
-        if self.peak == 'ID':
+        if self.peek == 'ID':
             name = self.consume('ID')
-        elif self.peak == '(':
+        elif self.peek == '(':
             # May be pointer to function?
             self.consume('(')
             if self.is_declaration_statement():
@@ -475,22 +525,22 @@ class CParser(RecursiveDescentParser):
             else:
                 self.error('Expected a name')
             # self.not_impl()
-            # raise NotImplementedError(str(self.peak))
+            # raise NotImplementedError(str(self.peek))
 
         # Now we have name, check for function decl:
         while True:
-            if self.peak == '(':
+            if self.peek == '(':
                 self.consume('(')
                 arguments = self.parse_function_arguments()
                 self.consume(')')
                 last_modifiers.append(('FUNCTION', arguments))
-            elif self.peak == '[':
+            elif self.peek == '[':
                 # Handle array type suffix:
                 self.consume('[')
-                if self.peak == '*':
+                if self.peek == '*':
                     # Handle VLA arrays:
                     amount = 'vla'
-                elif self.peak == ']':
+                elif self.peek == ']':
                     amount = None
                 else:
                     amount = self.parse_expression()
@@ -535,13 +585,13 @@ class CParser(RecursiveDescentParser):
 
     def is_declaration_statement(self):
         """ Determine whether we are facing a declaration or not """
-        if self.peak in self.storage_classes:
+        if self.peek in self.storage_classes:
             return True
-        elif self.peak in self.type_qualifiers:
+        elif self.peek in self.type_qualifiers:
             return True
-        elif self.peak in self.type_specifiers:
+        elif self.peek in self.type_specifiers:
             return True
-        elif self.peak in ('struct', 'union', 'enum', 'TYPE-ID'):
+        elif self.peek in ('struct', 'union', 'enum', 'TYPE-ID'):
             return True
         else:
             return False
@@ -563,11 +613,11 @@ class CParser(RecursiveDescentParser):
             '{': self.parse_compound_statement,
             ';': self.parse_empty_statement,
             }
-        if self.peak in m:
-            statement = m[self.peak]()
+        if self.peek in m:
+            statement = m[self.peek]()
         else:
             # Expression statement!
-            if self.peak == 'ID' and self.look_ahead(1).val == ':':
+            if self.peek == 'ID' and self.look_ahead(1).val == ':':
                 statement = self.parse_label()
             else:
                 expression = self.parse_expression()
@@ -592,7 +642,7 @@ class CParser(RecursiveDescentParser):
         statements = []
         location = self.consume('{').loc
         self.semantics.enter_compound_statement(location)
-        while self.peak != '}':
+        while self.peek != '}':
             statements.extend(self.parse_statement_or_declaration())
         self.consume('}')
         return self.semantics.on_compound_statement(statements, location)
@@ -673,19 +723,25 @@ class CParser(RecursiveDescentParser):
         """ Parse a for statement """
         location = self.consume('for').loc
         self.consume('(')
-        if self.peak == ';':
+        if self.peek == ';':
             initial = None
         else:
-            initial = self.parse_expression()
+            if self.is_declaration_statement():
+                ds = self.parse_decl_specifiers()
+                d = self.parse_declarator()
+                variable_declaration = self.parse_variable_declaration(ds, d)
+                initial = variable_declaration
+            else:
+                initial = self.parse_expression()
         self.consume(';')
 
-        if self.peak == ';':
+        if self.peek == ';':
             condition = None
         else:
             condition = self.parse_expression()
         self.consume(';')
 
-        if self.peak == ')':
+        if self.peek == ')':
             post = None
         else:
             post = self.parse_expression()
@@ -697,7 +753,7 @@ class CParser(RecursiveDescentParser):
     def parse_return_statement(self):
         """ Parse a return statement """
         location = self.consume('return').loc
-        if self.peak == ';':
+        if self.peek == ';':
             value = None
         else:
             value = self.parse_expression()
@@ -745,8 +801,8 @@ class CParser(RecursiveDescentParser):
     def parse_binop_with_precedence(self, priority):
         lhs = self.parse_primary_expression()
 
-        # print('prio=', prio, self.peak)
-        while self._binop_take(self.peak, priority):
+        # print('prio=', prio, self.peek)
+        while self._binop_take(self.peek, priority):
             op = self.consume()
             op_associativity, op_prio = self.prio_map[op.val]
             if op.val == '?':
@@ -769,20 +825,20 @@ class CParser(RecursiveDescentParser):
 
     def parse_primary_expression(self):
         """ Parse a primary expression """
-        if self.peak == 'ID':
+        if self.peek == 'ID':
             identifier = self.consume('ID')
             expr = self.semantics.on_variable_access(
                 identifier.val, identifier.loc)
-        elif self.peak == 'NUMBER':
+        elif self.peek == 'NUMBER':
             n = self.consume()
             expr = self.semantics.on_number(n.val, n.loc)
-        elif self.peak == 'CHAR':
+        elif self.peek == 'CHAR':
             n = self.consume()
             expr = self.semantics.on_char(n.val, n.loc)
-        elif self.peak == 'STRING':
+        elif self.peek == 'STRING':
             txt = self.consume()
             expr = self.semantics.on_string(txt.val, txt.loc)
-        elif self.peak in ['!', '*', '+', '-', '~', '&', '--', '++']:
+        elif self.peek in ['!', '*', '+', '-', '~', '&', '--', '++']:
             op = self.consume()
             if op.val in ['--', '++']:
                 operator = op.val + 'x'
@@ -790,13 +846,13 @@ class CParser(RecursiveDescentParser):
                 operator = op.val
             expr = self.parse_primary_expression()
             expr = self.semantics.on_unop(operator, expr, op.loc)
-        elif self.peak == '__builtin_va_start':
+        elif self.peek == '__builtin_va_start':
             location = self.consume('__builtin_va_start').loc
             self.consume('(')
             ap = self.parse_assignment_expression()
             self.consume(')')
             expr = self.semantics.on_builtin_va_start(ap, location)
-        elif self.peak == '__builtin_va_arg':
+        elif self.peek == '__builtin_va_arg':
             location = self.consume('__builtin_va_arg').loc
             self.consume('(')
             ap = self.parse_assignment_expression()
@@ -804,9 +860,17 @@ class CParser(RecursiveDescentParser):
             typ = self.parse_typename()
             self.consume(')')
             expr = self.semantics.on_builtin_va_arg(ap, typ, location)
-        elif self.peak == 'sizeof':
+        elif self.peek == '__builtin_offsetof':
+            location = self.consume('__builtin_offsetof').loc
+            self.consume('(')
+            typ = self.parse_typename()
+            self.consume(',')
+            member = self.consume('ID').val
+            self.consume(')')
+            expr = self.semantics.on_builtin_offsetof(typ, member, location)
+        elif self.peek == 'sizeof':
             location = self.consume('sizeof').loc
-            if self.peak == '(':
+            if self.peek == '(':
                 self.consume('(')
                 if self.is_declaration_statement():
                     typ = self.parse_typename()
@@ -817,7 +881,7 @@ class CParser(RecursiveDescentParser):
             else:
                 sizeof_expr = self.parse_primary_expression()
                 expr = self.semantics.on_sizeof(sizeof_expr, location)
-        elif self.peak == '(':
+        elif self.peek == '(':
             loc = self.consume('(').loc
             # Is this a type cast?
             if self.is_declaration_statement():
@@ -831,25 +895,25 @@ class CParser(RecursiveDescentParser):
                 expr = self.parse_expression()
                 self.consume(')')
         else:
-            self.not_impl(self.peak)
+            self.not_impl(self.peek)
 
         # Postfix operations (have the highest precedence):
-        while self.peak in ['--', '++', '[', '.', '->', '(']:
-            if self.peak in ['--', '++']:
+        while self.peek in ['--', '++', '[', '.', '->', '(']:
+            if self.peek in ['--', '++']:
                 op = self.consume()
                 expr = self.semantics.on_unop('x' + op.val, expr, op.loc)
-            elif self.peak == '[':
+            elif self.peek == '[':
                 location = self.consume('[').loc
                 index = self.parse_expression()
                 self.consume(']')
                 expr = self.semantics.on_array_index(expr, index, location)
-            elif self.peak == '(':
+            elif self.peek == '(':
                 expr = self.parse_call(expr)
-            elif self.peak == '.':
+            elif self.peek == '.':
                 location = self.consume('.').loc
                 field = self.consume('ID').val
                 expr = self.semantics.on_field_select(expr, field, location)
-            elif self.peak == '->':
+            elif self.peek == '->':
                 location = self.consume('->').loc
                 field = self.consume('ID').val
                 # Dereference pointer:
@@ -863,9 +927,9 @@ class CParser(RecursiveDescentParser):
         """ Parse a function call """
         location = self.consume('(').loc
         args = []
-        while self.peak != ')':
+        while self.peek != ')':
             args.append(self.parse_assignment_expression())
-            if self.peak != ')':
+            if self.peek != ')':
                 self.consume(',')
         expr = self.semantics.on_call(
             callee, args, location)
@@ -884,7 +948,7 @@ class CParser(RecursiveDescentParser):
         return tok
 
     @property
-    def peak(self):
+    def peek(self):
         """ Look at the next token to parse without popping it """
         if self.token:
             # Also implement lexer hack here:
