@@ -18,6 +18,7 @@ class RecursiveDescentParser:
         self.token = None  # The current token under cursor
         self.tokens = None  # Iterable of tokens
         self._look_ahead = []
+        self._last_loc = None
 
     def init_lexer(self, tokens):
         """ Initialize the parser with the given tokens (an iterator) """
@@ -28,7 +29,13 @@ class RecursiveDescentParser:
     def error(self, msg, loc=None):
         """ Raise an error at the current location """
         if loc is None:
-            loc = self.token.loc
+            if self.token is None:
+                if self._last_loc is None:
+                    loc = None
+                else:
+                    loc = self._last_loc
+            else:
+                loc = self.token.loc
         raise CompilerError(msg, loc)
 
     @property
@@ -43,21 +50,21 @@ class RecursiveDescentParser:
         If typ is not given, consume the next token.
         """
         if typ is None:
-            typ = self.peak
+            typ = self.peek
 
         expected_types = typ if isinstance(typ, (list, tuple, set)) else [typ]
 
-        if self.peak in expected_types:
+        if self.peek in expected_types:
             return self.next_token()
         else:
             expected = make_comma_or(expected_types)
             self.error(
-                'Expected {0}, got "{1}"'.format(expected, self.peak))
+                'Expected {0}, got "{1}"'.format(expected, self.peek))
 
     def has_consumed(self, typ):
         """ Checks if the look-ahead token is of type typ, and if so
             eats the token and returns true """
-        if self.peak == typ:
+        if self.peek == typ:
             self.consume()
             return True
         return False
@@ -69,6 +76,10 @@ class RecursiveDescentParser:
             self.token = self._look_ahead.pop(0)
         else:
             self.token = next(self.tokens, None)
+
+        if self.token is None and tok is not None:
+            self._last_loc = tok.loc
+
         return tok
 
     def not_impl(self, msg=''):  # pragma: no cover
@@ -76,7 +87,7 @@ class RecursiveDescentParser:
         raise CompilerError('Not implemented ' + msg, loc=self.token.loc)
 
     @property
-    def peak(self):
+    def peek(self):
         """ Look at the next token to parse without popping it """
         if self.token:
             return self.token.typ
@@ -95,4 +106,4 @@ class RecursiveDescentParser:
 
     @property
     def at_end(self):
-        return self.peak is None
+        return self.peek is None
