@@ -2,7 +2,8 @@ from .clean import CleanPass
 from .mem2reg import Mem2RegPromotor
 from .cse import CommonSubexpressionEliminationPass
 from .constantfolding import ConstantFolder
-from .transform import RemoveAddZeroPass, LoadAfterStorePass
+from .load_after_store import LoadAfterStorePass
+from .transform import RemoveAddZeroPass
 from .transform import DeleteUnusedInstructionsPass
 from .transform import ModulePass, FunctionPass, BlockPass, InstructionPass
 
