@@ -1,6 +1,6 @@
 
 from .. import ir
-from .transform import FunctionPass:
+from .transform import FunctionPass
 
 
 def only_used_by_phis(phi):
@@ -14,6 +14,7 @@ class OnlyPhiPass(FunctionPass):
     are left in a circle. This pass gets rid of these.
     """
     def on_function(self, function):
+        raise NotImplementedError()
         # Gather all phi nodes:
         phis = set()
         for block in function:
@@ -27,4 +28,3 @@ class OnlyPhiPass(FunctionPass):
 
             if all(isinstance(p, ir.Phi) for p in phi.used_by):
                 pass
-
